@@ -33,12 +33,12 @@
 #include <glib/gstdio.h>
 
 #include <libtracker-common/tracker-common.h>
+
 #include <libtracker-data/tracker-db-config.h>
 #include <libtracker-data/tracker-db-journal.h>
 #include <libtracker-data/tracker-db-manager.h>
-#include <libtracker-miner/tracker-miner.h>
 
-#include "tracker-miner-files-index-client.h"
+#include <libtracker-miner/tracker-miner.h>
 
 #define ABOUT	  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -303,7 +303,7 @@ main (int argc, char **argv)
 	 */
 	if (kill_option != TERM_NONE ||
 	    terminate_option != TERM_NONE ||
-	    (!start && !remove_config && !reindex_mime_types && 
+	    (!start && !remove_config && !reindex_mime_types &&
 	     !print_version && !index_file)) {
 		pids = get_pids ();
 		str = g_strdup_printf (g_dngettext (NULL,
@@ -434,15 +434,23 @@ main (int argc, char **argv)
 		tracker_db_journal_set_rotating ((chunk_size_mb != -1),
 		                                 chunk_size, rotate_to);
 
-		tracker_db_journal_init (NULL, FALSE);
 
 		g_object_unref (db_config);
 
 		/* Clean up (select_cache_size and update_cache_size don't matter here) */
-		if (!tracker_db_manager_init (TRACKER_DB_MANAGER_REMOVE_ALL, NULL, FALSE, 100, 100, NULL, NULL)) {
+		if (!tracker_db_manager_init (TRACKER_DB_MANAGER_REMOVE_ALL,
+		                              NULL,
+		                              FALSE,
+		                              100,
+		                              100,
+		                              NULL,
+		                              NULL,
+		                              NULL)) {
 			return EXIT_FAILURE;
 		}
 
+		tracker_db_journal_init (NULL, FALSE);
+
 		tracker_db_manager_remove_all (hard_reset);
 		tracker_db_manager_shutdown ();
 		tracker_db_journal_shutdown ();
@@ -503,9 +511,6 @@ main (int argc, char **argv)
 		TrackerMinerManager *manager;
 		GSList *miners, *l;
 
-		manager = tracker_miner_manager_new ();
-		miners = tracker_miner_manager_get_available (manager);
-
 		if (hard_reset || soft_reset) {
 			g_print ("%s\n", _("Waiting one second before starting miners…"));
 
@@ -515,6 +520,9 @@ main (int argc, char **argv)
 
 		g_print ("%s\n", _("Starting miners…"));
 
+		manager = tracker_miner_manager_new ();
+		miners = tracker_miner_manager_get_available (manager);
+
 		/* Get the status of all miners, this will start all
 		 * miners not already running.
 		 */
