@@ -1,6 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -51,8 +50,12 @@ TrackerConfig *tracker_config_new           (void);
 gboolean       tracker_config_save          (TrackerConfig *config);
 
 gint           tracker_config_get_verbosity (TrackerConfig *config);
+gint           tracker_config_get_max_bytes (TrackerConfig *config);
+
 void           tracker_config_set_verbosity (TrackerConfig *config,
                                              gint           value);
+void           tracker_config_set_max_bytes (TrackerConfig *config,
+                                             gint           value);
 
 G_END_DECLS
 
