@@ -49,7 +49,6 @@
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-albumart.h"
-#include "tracker-dbus.h"
 
 /* We mmap the beginning of the file and read separately the last 128
  * bytes for id3v1 tags. While these are probably cornercases the
@@ -152,8 +151,6 @@ typedef struct {
 	size_t size;
 	size_t id3v2_size;
 
-	guint32 duration;
-
 	const gchar *title;
 	const gchar *performer;
 	gchar *performer_uri;
@@ -412,7 +409,7 @@ static const guint freq_mask = 0xC0000;
 static const guint ch_mask = 0xC0000000;
 static const guint pad_mask = 0x20000;
 
-static guint bitrate_table[16][6] = {
+static gint bitrate_table[16][6] = {
 	{   0,   0,   0,   0,   0,   0 },
 	{  32,  32,  32,  32,   8,   8 },
 	{  64,  48,  40,  48,  16,  16 },
@@ -434,7 +431,8 @@ static guint bitrate_table[16][6] = {
 static gint freq_table[4][3] = {
 	{ 44100, 22050, 11025 },
 	{ 48000, 24000, 12000 },
-	{ 32000, 16000, 8000  }
+	{ 32000, 16000,  8000 },
+	{    -1,     -1,   -1 }
 };
 
 static gint spf_table[6] = {
@@ -825,7 +823,10 @@ get_id3 (const gchar *data,
 	return TRUE;
 }
 
-
+/*
+ * For the MP3 frame header description, see
+ * http://www.mp3-tech.org/programmer/frame_header.html
+ */
 static gboolean
 mp3_parse_header (const gchar          *data,
                   size_t                size,
@@ -840,11 +841,11 @@ mp3_parse_header (const gchar          *data,
 	gint spfp8 = 0;
 	guint padsize = 0;
 	gint idx_num = 0;
-	guint bitrate = 0;
+	gint bitrate = 0;
 	guint avg_bps = 0;
 	gint vbr_flag = 0;
 	guint length = 0;
-	guint sample_rate = 0;
+	gint sample_rate = 0;
 	guint frame_size;
 	guint frames = 0;
 	size_t pos = 0;
@@ -897,8 +898,10 @@ mp3_parse_header (const gchar          *data,
 	/* We assume mpeg version, layer and channels are constant in frames */
 	do {
 		frames++;
+
 		bitrate = 1000 * bitrate_table[(header & bitrate_mask) >> 20][idx_num];
 
+		/* Skip frame headers with bitrate index '0000' (free) or '1111' (bad) */
 		if (bitrate <= 0) {
 			frames--;
 			return FALSE;
@@ -906,7 +909,13 @@ mp3_parse_header (const gchar          *data,
 
 		sample_rate = freq_table[(header & freq_mask) >> 18][mpeg_ver - 1];
 
-		frame_size = spfp8 * bitrate / (sample_rate ? sample_rate : 1) + padsize*((header & pad_mask) >> 17);
+		/* Skip frame headers with frequency index '11' (reserved) */
+		if (sample_rate <= 0) {
+			frames--;
+			return FALSE;
+		}
+
+		frame_size = spfp8 * bitrate / sample_rate + padsize*((header & pad_mask) >> 17);
 		avg_bps += bitrate / 1000;
 
 		pos += frame_size;
@@ -950,19 +959,18 @@ mp3_parse_header (const gchar          *data,
 
 	avg_bps /= frames;
 
-	if (filedata->duration == 0) {
-		if ((!vbr_flag && frames > VBR_THRESHOLD) || (frames > MAX_FRAMES_SCAN)) {
-			/* If not all frames scanned
-			 * Note that bitrate is always > 0, checked before */
-			length = (filedata->size - filedata->id3v2_size) / (avg_bps ? avg_bps : bitrate) / 125;
-		} else{
-			length = spfp8 * 8 * frames / (sample_rate ? sample_rate : 0xFFFFFFFF);
-		}
-
-		tracker_sparql_builder_predicate (metadata, "nfo:duration");
-		tracker_sparql_builder_object_int64 (metadata, length);
+	if ((!vbr_flag && frames > VBR_THRESHOLD) || (frames > MAX_FRAMES_SCAN)) {
+		/* If not all frames scanned
+		 * Note that bitrate is always > 0, checked before */
+		length = (filedata->size - filedata->id3v2_size) / (avg_bps ? avg_bps : bitrate) / 125;
+	} else {
+		/* Note that sample_rate is always > 0, checked before */
+		length = spfp8 * 8 * frames / sample_rate;
 	}
 
+	tracker_sparql_builder_predicate (metadata, "nfo:duration");
+	tracker_sparql_builder_object_int64 (metadata, length);
+
 	tracker_sparql_builder_predicate (metadata, "nfo:sampleRate");
 	tracker_sparql_builder_object_int64 (metadata, sample_rate);
 	tracker_sparql_builder_predicate (metadata, "nfo:averageBitrate");
@@ -2298,10 +2306,10 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_object_iri (metadata, md.album_uri);
 	}
 
-	tracker_guarantee_date_from_file_mtime (metadata,
-	                                        "nie:contentCreated",
-	                                        md.recording_time,
-	                                        uri);
+	if (md.recording_time) {
+		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
+		tracker_sparql_builder_object_unvalidated (metadata, md.recording_time);
+	}
 
 	if (md.genre) {
 		tracker_sparql_builder_predicate (metadata, "nfo:genre");
