@@ -27,12 +27,12 @@
 
 #include <gio/gunixoutputstream.h>
 #include <gio/gunixinputstream.h>
+#include <gio/gunixfdlist.h>
 
 #include <libtracker-common/tracker-common.h>
 
 #include <libtracker-extract/tracker-extract.h>
 
-#include "tracker-dbus.h"
 #include "tracker-extract.h"
 #include "tracker-main.h"
 #include "tracker-marshal.h"
@@ -48,6 +48,26 @@
 #define UNKNOWN_METHOD_MESSAGE "Method \"%s\" with signature \"%s\" on " \
                                "interface \"%s\" doesn't exist, expected \"%s\""
 
+static const gchar introspection_xml[] =
+  "<node>"
+  "  <interface name='org.freedesktop.Tracker1.Extract'>"
+  "    <method name='GetPid'>"
+  "      <arg type='i' name='value' direction='out' />"
+  "    </method>"
+  "    <method name='GetMetadata'>"
+  "      <arg type='s' name='uri' direction='in' />"
+  "      <arg type='s' name='mime' direction='in' />"
+  "      <arg type='s' name='preupdate' direction='out' />"
+  "      <arg type='s' name='embedded' direction='out' />"
+  "    </method>"
+  "    <method name='GetMetadataFast'>"
+  "      <arg type='s' name='uri' direction='in' />"
+  "      <arg type='s' name='mime' direction='in' />"
+  "      <arg type='h' name='fd' direction='in' />"
+  "    </method>"
+  "  </interface>"
+  "</node>";
+
 #define TRACKER_EXTRACT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_EXTRACT, TrackerExtractPrivate))
 
 extern gboolean debug;
@@ -58,6 +78,10 @@ typedef struct {
 	gboolean disable_shutdown;
 	gboolean force_internal_extractors;
 	gboolean disable_summary_on_finalize;
+	GDBusConnection *d_connection;
+	GDBusNodeInfo *introspection_data;
+	guint registration_id;
+	guint own_id;
 } TrackerExtractPrivate;
 
 typedef struct {
@@ -214,12 +238,12 @@ load_modules (const gchar  *force_module,
 	gboolean success;
 	const gchar *extractors_dir;
 
-        extractors_dir = g_getenv ("TRACKER_EXTRACTORS_DIR");
-        if (G_LIKELY (extractors_dir == NULL)) {
-                extractors_dir = TRACKER_EXTRACTORS_DIR;
-        } else {
-                g_message ("Extractor modules directory is '%s' (set in env)", extractors_dir);
-        }
+	extractors_dir = g_getenv ("TRACKER_EXTRACTORS_DIR");
+	if (G_LIKELY (extractors_dir == NULL)) {
+		extractors_dir = TRACKER_EXTRACTORS_DIR;
+	} else {
+		g_message ("Extractor modules directory is '%s' (set in env)", extractors_dir);
+	}
 
 	dir = g_dir_open (extractors_dir, 0, &error);
 
@@ -369,11 +393,11 @@ tracker_extract_new (gboolean     disable_shutdown,
 static gboolean
 get_file_metadata (TrackerExtract         *extract,
                    TrackerDBusRequest     *request,
-                   DBusGMethodInvocation  *context,
+                   GDBusMethodInvocation  *invocation,
                    const gchar            *uri,
                    const gchar            *mime,
-		   TrackerSparqlBuilder  **preupdate_out,
-		   TrackerSparqlBuilder  **statements_out)
+                   TrackerSparqlBuilder  **preupdate_out,
+                   TrackerSparqlBuilder  **statements_out)
 {
 	TrackerExtractPrivate *priv;
 	TrackerSparqlBuilder *statements, *preupdate;
@@ -629,9 +653,9 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
 		}
 
 		tracker_dbus_request_info (request, "%s",
-					   preupdate_str ? preupdate_str : "");
+		                           preupdate_str ? preupdate_str : "");
 		tracker_dbus_request_info (request, "%s",
-					   statements_str ? statements_str : "");
+		                           statements_str ? statements_str : "");
 
 		g_object_unref (statements);
 		g_object_unref (preupdate);
@@ -640,15 +664,15 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
 	tracker_dbus_request_end (request, NULL);
 }
 
-void
-tracker_extract_get_pid (TrackerExtract         *object,
-                         DBusGMethodInvocation  *context,
-                         GError                **error)
+static void
+handle_method_call_get_pid (TrackerExtract        *object,
+                            GDBusMethodInvocation *invocation,
+                            GVariant              *parameters)
 {
 	TrackerDBusRequest *request;
 	pid_t value;
 
-	request = tracker_dbus_g_request_begin (context,
+	request = tracker_g_dbus_request_begin (invocation,
 	                                        "%s()",
 	                                        __FUNCTION__);
 
@@ -658,25 +682,27 @@ tracker_extract_get_pid (TrackerExtract         *object,
 	                            value);
 
 	tracker_dbus_request_end (request, NULL);
-	dbus_g_method_return (context, value);
+
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(i)", (gint) value));
 }
 
-void
-tracker_extract_get_metadata (TrackerExtract         *object,
-                              const gchar            *uri,
-                              const gchar            *mime,
-                              DBusGMethodInvocation  *context,
-                              GError                **error)
+static void
+handle_method_call_get_metadata (TrackerExtract        *object,
+                                 GDBusMethodInvocation *invocation,
+                                 GVariant              *parameters)
 {
 	TrackerDBusRequest *request;
 	TrackerExtractPrivate *priv;
 	TrackerSparqlBuilder *sparql, *preupdate;
 	gboolean extracted = FALSE;
+	const gchar *uri = NULL, *mime = NULL;
 
+	g_variant_get (parameters, "(&s&s)", &uri, &mime);
 
-	tracker_dbus_async_return_if_fail (uri != NULL, context);
+	tracker_gdbus_async_return_if_fail (uri != NULL, invocation);
 
-	request = tracker_dbus_g_request_begin (context,
+	request = tracker_g_dbus_request_begin (invocation,
 	                                        "%s(uri:'%s', mime:%s)",
 	                                        __FUNCTION__,
 	                                        uri,
@@ -692,7 +718,13 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 		alarm (MAX_EXTRACT_TIME);
 	}
 
-	extracted = get_file_metadata (object, request, context, uri, mime, &preupdate, &sparql);
+	extracted = get_file_metadata (object,
+	                               request,
+	                               invocation,
+	                               uri,
+	                               mime,
+	                               &preupdate,
+	                               &sparql);
 
 	if (extracted) {
 		tracker_dbus_request_end (request, NULL);
@@ -704,11 +736,13 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 				preupdate_str = tracker_sparql_builder_get_result (preupdate);
 			}
 
-			dbus_g_method_return (context,
-			                      preupdate_str ? preupdate_str : "",
-			                      tracker_sparql_builder_get_result (sparql));
+			g_dbus_method_invocation_return_value (invocation,
+			                                       g_variant_new ("(ss)",
+			                                                      preupdate_str ? preupdate_str : "",
+			                                                      tracker_sparql_builder_get_result (sparql)));
 		} else {
-			dbus_g_method_return (context, "", "");
+			g_dbus_method_invocation_return_value (invocation,
+			                                       g_variant_new ("(ss)", "", ""));
 		}
 
 		g_object_unref (sparql);
@@ -721,7 +755,7 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 		                            uri,
 		                            mime);
 		tracker_dbus_request_end (request, actual_error);
-		dbus_g_method_return_error (context, actual_error);
+		g_dbus_method_invocation_return_gerror (invocation, actual_error);
 		g_error_free (actual_error);
 	}
 
@@ -732,69 +766,47 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 }
 
 static void
-get_metadata_fast (TrackerExtract *object,
-                   DBusConnection *connection,
-                   DBusMessage    *message)
+handle_method_call_get_metadata_fast (TrackerExtract        *object,
+                                      GDBusMethodInvocation *invocation,
+                                      GVariant              *parameters)
 {
 	TrackerDBusRequest *request;
-	const gchar *expected_signature;
 	TrackerExtractPrivate *priv;
-	DBusError dbus_error;
-	DBusMessage *reply;
+	GDBusMessage *reply;
 	const gchar *uri, *mime;
-	int fd;
+	int fd, index_fd;
 	GOutputStream *unix_output_stream;
 	GOutputStream *buffered_output_stream;
 	GDataOutputStream *data_output_stream;
 	GError *error = NULL;
 	TrackerSparqlBuilder *sparql, *preupdate;
 	gboolean extracted = FALSE;
+	GDBusMessage *method_message;
+	GDBusConnection *connection;
+	GUnixFDList *fd_list;
 
+	connection = g_dbus_method_invocation_get_connection (invocation);
+	method_message = g_dbus_method_invocation_get_message (invocation);
 
-	expected_signature = DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_UNIX_FD_AS_STRING;
+	g_variant_get (parameters, "(&s&sh)", &uri, &mime, &index_fd);
 
-	if (g_strcmp0 (dbus_message_get_signature (message), expected_signature)) {
-		gchar *error_message = g_strdup_printf (UNKNOWN_METHOD_MESSAGE,
-		                                        "GetMetadataFast",
-		                                        dbus_message_get_signature (message),
-		                                        dbus_message_get_interface (message),
-		                                        expected_signature);
-		reply = dbus_message_new_error (message,
-		                                DBUS_ERROR_UNKNOWN_METHOD,
-		                                error_message);
-		dbus_connection_send (connection, reply, NULL);
+	fd_list = g_dbus_message_get_unix_fd_list (method_message);
 
-		dbus_message_unref (reply);
-		g_free (error_message);
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s(uri:'%s', mime:%s)",
+	                                        __FUNCTION__,
+	                                        uri,
+	                                        mime);
 
-		return;
+	if ((fd = g_unix_fd_list_get (fd_list, index_fd, &error)) == -1) {
+		tracker_dbus_request_end (request, error);
+		reply = g_dbus_message_new_method_error_literal (method_message,
+		                                                 TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
+		                                                 error->message);
+		g_error_free (error);
+		goto bail_out;
 	}
 
-	dbus_error_init (&dbus_error);
-
-	dbus_message_get_args (message,
-	                       &dbus_error,
-	                       DBUS_TYPE_STRING, &uri,
-	                       DBUS_TYPE_STRING, &mime,
-	                       DBUS_TYPE_UNIX_FD, &fd,
-	                       DBUS_TYPE_INVALID);
-
-	if (dbus_error_is_set (&dbus_error)) {
-		reply = dbus_message_new_error (message, dbus_error.name, dbus_error.message);
-		dbus_connection_send (connection, reply, NULL);
-
-		dbus_message_unref (reply);
-		dbus_error_free (&dbus_error);
-
-		return;
-	}
-
-	request = tracker_dbus_request_begin (dbus_message_get_sender (message),
-	                                      "%s(uri:'%s', mime:%s)",
-	                                      __FUNCTION__,
-	                                      uri,
-	                                      mime);
-
 	tracker_dbus_request_debug (request,
 	                            "  Resetting shutdown timeout");
 
@@ -857,64 +869,163 @@ get_metadata_fast (TrackerExtract *object,
 
 		if (error) {
 			tracker_dbus_request_end (request, error);
-			reply = dbus_message_new_error (message,
-			                                TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
-			                                error->message);
-			dbus_connection_send (connection, reply, NULL);
-			dbus_message_unref (reply);
+			reply = g_dbus_message_new_method_error_literal (method_message,
+			                                                 TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
+			                                                 error->message);
 			g_error_free (error);
 		} else {
 			tracker_dbus_request_end (request, NULL);
-			reply = dbus_message_new_method_return (message);
-			dbus_connection_send (connection, reply, NULL);
-			dbus_message_unref (reply);
+			reply = g_dbus_message_new_method_reply (method_message);
 		}
 	} else {
 		error = g_error_new (TRACKER_DBUS_ERROR, 0,
 		                     "Could not get any metadata for uri:'%s' and mime:'%s'", uri, mime);
 		tracker_dbus_request_end (request, error);
-		reply = dbus_message_new_error (message,
-		                                TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
-		                                error->message);
-		close (fd);
-		dbus_connection_send (connection, reply, NULL);
-		dbus_message_unref (reply);
+		reply = g_dbus_message_new_method_error_literal (method_message,
+		                                                 TRACKER_EXTRACT_SERVICE ".GetMetadataFastError",
+		                                                 error->message);
 		g_error_free (error);
+		close (fd);
 	}
 
+bail_out:
+
+	g_dbus_connection_send_message (connection, reply,
+	                                G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+	                                NULL, NULL);
+
+	g_object_unref (fd_list);
+	g_object_unref (reply);
+
 	if (!priv->disable_shutdown) {
 		/* Unset alarm so the extractor doesn't die when it's idle */
 		alarm (0);
 	}
 }
 
-DBusHandlerResult
-tracker_extract_connection_filter (DBusConnection *connection,
-                                   DBusMessage    *message,
-                                   void           *user_data)
+static void
+handle_method_call (GDBusConnection       *connection,
+                    const gchar           *sender,
+                    const gchar           *object_path,
+                    const gchar           *interface_name,
+                    const gchar           *method_name,
+                    GVariant              *parameters,
+                    GDBusMethodInvocation *invocation,
+                    gpointer               user_data)
 {
-	TrackerExtract *extract;
+	TrackerExtract *extract = user_data;
 
-	g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
-	g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
+	if (g_strcmp0 (method_name, "GetPid") == 0) {
+		handle_method_call_get_pid (extract, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetMetadataFast") == 0) {
+		handle_method_call_get_metadata_fast (extract, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetMetadata") == 0) {
+		handle_method_call_get_metadata (extract, invocation, parameters);
+	} else {
+		g_assert_not_reached ();
+	}
+}
 
-	if (g_strcmp0 (TRACKER_EXTRACT_PATH, dbus_message_get_path (message))) {
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+static GVariant *
+handle_get_property (GDBusConnection  *connection,
+                     const gchar      *sender,
+                     const gchar      *object_path,
+                     const gchar      *interface_name,
+                     const gchar      *property_name,
+                     GError          **error,
+                     gpointer          user_data)
+{
+	g_assert_not_reached ();
+	return NULL;
+}
+
+static gboolean
+handle_set_property (GDBusConnection  *connection,
+                     const gchar      *sender,
+                     const gchar      *object_path,
+                     const gchar      *interface_name,
+                     const gchar      *property_name,
+                     GVariant         *value,
+                     GError          **error,
+                     gpointer          user_data)
+{
+	g_assert_not_reached ();
+	return TRUE;
+}
+
+void
+tracker_extract_dbus_start (TrackerExtract *extract)
+{
+	TrackerExtractPrivate *priv;
+	GError *error = NULL;
+	GDBusInterfaceVTable interface_vtable = {
+		handle_method_call,
+		handle_get_property,
+		handle_set_property
+	};
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	priv->d_connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+
+	if (!priv->d_connection) {
+		g_critical ("Could not connect to the D-Bus session bus, %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return;
+	}
+
+	priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+
+	priv->own_id = g_bus_own_name_on_connection (priv->d_connection,
+	                                             TRACKER_EXTRACT_SERVICE,
+	                                             G_BUS_NAME_OWNER_FLAGS_NONE,
+	                                             NULL, NULL, NULL, NULL);
+
+	g_message ("Registering D-Bus object...");
+	g_message ("  Path:'" TRACKER_EXTRACT_PATH "'");
+	g_message ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (extract));
+
+	priv->registration_id =
+		g_dbus_connection_register_object (priv->d_connection,
+		                                   TRACKER_EXTRACT_PATH,
+		                                   priv->introspection_data->interfaces[0],
+		                                   &interface_vtable,
+		                                   extract,
+		                                   NULL,
+		                                   &error);
+
+	if (error) {
+		g_critical ("Could not register the D-Bus object "TRACKER_EXTRACT_PATH", %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return;
+	}
+}
+
+void
+tracker_extract_dbus_stop (TrackerExtract *extract)
+{
+	TrackerExtractPrivate *priv;
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	if (priv->own_id != 0) {
+		g_bus_unown_name (priv->own_id);
+	}
+
+	if (priv->registration_id != 0) {
+		g_dbus_connection_unregister_object (priv->d_connection,
+		                                     priv->registration_id);
+	}
+
+	if (priv->introspection_data) {
+		g_dbus_node_info_unref (priv->introspection_data);
+	}
+
+	if (priv->d_connection) {
+		g_object_unref (priv->d_connection);
 	}
-
-	if (g_strcmp0 (TRACKER_EXTRACT_INTERFACE, dbus_message_get_interface (message))) {
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-	}
-
-	/* Only check if the user_data is our TrackerExtract AFTER having checked that
-	 * the message matches expected path and interface. */
-	extract = user_data;
-	g_return_val_if_fail (TRACKER_IS_EXTRACT (extract), DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
-
-	if (!g_strcmp0 ("GetMetadataFast", dbus_message_get_member (message))) {
-		get_metadata_fast (extract, connection, message);
-		return DBUS_HANDLER_RESULT_HANDLED;
-	}
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
