@@ -21,6 +21,92 @@ XDG_CONFIG_HOME_DIR = os.path.join (cfg.TEST_TMP_DIR, "xdg-config-home")
 
 REASONABLE_TIMEOUT = 30
 
+class UnableToBootException (Exception):
+    pass
+
+class TrackerStoreLifeCycle ():
+
+    def __init__ (self):
+        self.timeout_id = 0
+
+    def start (self):
+        """
+        call this method to start and instance of tracker-store. It will return when the store is ready
+        """
+        self.loop = gobject.MainLoop()
+        dbus_loop = DBusGMainLoop(set_as_default=True)
+        self.bus = dbus.SessionBus (dbus_loop)
+
+        obj = self.bus.get_object ("org.freedesktop.DBus",
+                                   "/org/freedesktop/DBus")
+        self.admin = dbus.Interface (obj, dbus_interface="org.freedesktop.DBus")
+        if (self.admin.NameHasOwner (cfg.TRACKER_BUSNAME)):
+            raise Exception ("Store is already running! kill it before starting this one")
+
+        self.name_owner_match = self.bus.add_signal_receiver (self.__name_owner_changed_cb,
+                                                         signal_name="NameOwnerChanged",
+                                                         path="/org/freedesktop/DBus",
+                                                         dbus_interface="org.freedesktop.DBus")
+        self.store_proc = self.__start_tracker_store ()
+        
+        # It should step out of this loop when the miner is visible in DBus
+        self.loop.run ()
+
+        tracker = self.bus.get_object (cfg.TRACKER_BUSNAME, cfg.TRACKER_OBJ_PATH)
+        tracker_status = self.bus.get_object (cfg.TRACKER_BUSNAME, cfg.TRACKER_STATUS_OBJ_PATH)
+        self.status_iface = dbus.Interface (tracker_status, dbus_interface=cfg.STATUS_IFACE)
+        print "[store] booting..."
+        self.status_iface.Wait ()
+        print "[store] ready."
+
+        
+    def stop (self):
+        self.__stop_tracker_store ()
+        # It should step out of this loop when the miner disappear from the bus
+        self.timeout_id = glib.timeout_add_seconds (REASONABLE_TIMEOUT, self.__timeout_on_idle)
+        self.loop.run ()
+
+        print "[store] stop."
+        # Disconnect the signals of the next start we get duplicated messages
+        self.bus._clean_up_signal_match (self.name_owner_match)
+
+    def kill (self):
+        self.store_proc.kill ()
+        self.loop.run ()
+        # Name owner changed cb should take us out from this loop
+
+        print "[store] killed."
+        self.bus._clean_up_signal_match (self.name_owner_match)
+
+    def __timeout_on_idle (self):
+        print "[store] Timeout waiting... asumming idle."
+        self.loop.quit ()
+        return False
+
+    def __name_owner_changed_cb (self, name, old_owner, new_owner):
+        if name == cfg.TRACKER_BUSNAME:
+            if old_owner == '' and new_owner != '':
+                print "[store] appears in the bus"
+            elif old_owner != ''  and new_owner == '':
+                print "[store] disappears from the bus"
+            else:
+                print "[store] name change %s -> %s" % (old_owner, new_owner)
+            self.loop.quit ()
+
+    def __start_tracker_store (self):
+        tracker_binary = os.path.join (cfg.EXEC_PREFIX, "tracker-store")
+        tracker = [tracker_binary]
+        # The env variables can be passed as parameters!
+        FNULL = open('/dev/null', 'w')
+        return subprocess.Popen (tracker, stdout=FNULL, stderr=FNULL)
+
+    def __stop_tracker_store (self):
+        #control_binary = os.path.join (cfg.BINDIR, "tracker-control")
+        #FNULL = open('/dev/null', 'w')
+        #subprocess.call ([control_binary, "-t"], stdout=FNULL)
+        self.store_proc.terminate ()
+
+
 class TrackerMinerFsLifeCycle():
     """
     Starts and monitors the miner-fs life cycle
@@ -74,17 +160,17 @@ class TrackerMinerFsLifeCycle():
 
     def wait_for_idle (self, timeout=REASONABLE_TIMEOUT):
         # The signal is already connected
-        print "Waiting for Idle"
+        print "[miner-fs] waiting for Idle"
         self.timeout_id = glib.timeout_add_seconds (timeout, self.__timeout_on_idle)
         self.loop.run ()
 
     def __timeout_on_idle (self):
-        print "Timeout... asumming idle"
+        print "[miner-fs] timeout... asumming idle"
         self.loop.quit ()
         return False
 
     def __minerfs_status_cb (self, status, handle):
-        print "Miner status is now", status.encode ("utf-8")
+        print "[miner-fs] status is now", status.encode ("utf-8")
         if (status == "Idle"):
             if (self.timeout_id != 0):
                 glib.source_remove (self.timeout_id)
@@ -94,7 +180,12 @@ class TrackerMinerFsLifeCycle():
 
     def __name_owner_changed_cb (self, name, old_owner, new_owner):
         if name == cfg.MINERFS_BUSNAME:
-            print "Miner name change %s -> %s" % (old_owner, new_owner)
+            if old_owner == '' and new_owner != '':
+                print "[miner-fs] appears in the bus"
+            elif old_owner != ''  and new_owner == '':
+                print "[miner-fs] disappears from the bus"
+            else:
+                print "[miner-fs] name change %s -> %s" % (old_owner, new_owner)
             self.loop.quit ()
 
     def __start_tracker_miner_fs (self):
@@ -167,12 +258,12 @@ class TrackerWritebackLifeCycle():
 
 class TrackerSystemAbstraction:
 
-    def set_up_environment (self, confdir):
+    def set_up_environment (self, confdir, ontodir):
         """
         Sets up the XDG_*_HOME variables and make sure the directories exist
         """
         for var, directory in TEST_ENV_VARS.iteritems ():
-            print "Setting %s - %s" %(var, directory)
+            print "[Conf] Setting %s - %s" %(var, directory)
             self.__recreate_directory (directory)
             os.environ [var] = directory
 
@@ -183,10 +274,14 @@ class TrackerSystemAbstraction:
             self.__recreate_directory (XDG_CONFIG_HOME_DIR)
             shutil.copytree (os.path.join (confdir, "tracker"),
                              os.path.join (XDG_CONFIG_HOME_DIR, "tracker"))
-            print "Setting %s - %s" % ("XDG_CONFIG_HOME", XDG_CONFIG_HOME_DIR)
-            print "  taking configuration from", confdir
+            print "[Conf] Setting %s - %s" % ("XDG_CONFIG_HOME", XDG_CONFIG_HOME_DIR)
+            print "[Conf]   taking conf from", confdir
             os.environ ["XDG_CONFIG_HOME"] = XDG_CONFIG_HOME_DIR
 
+        if ontodir:
+            print "[Conf] Setting %s - %s" % ("TRACKER_DB_ONTOLOGIES_DIR", ontodir)
+            os.environ ["TRACKER_DB_ONTOLOGIES_DIR"] = ontodir
+
     def unset_up_environment (self):
         """
         Unset the XDG_*_HOME variables from the environment
@@ -197,28 +292,46 @@ class TrackerSystemAbstraction:
                 
         if (os.environ.has_key ("XDG_CONFIG_HOME")):
             del os.environ ["XDG_CONFIG_HOME"]
-        
 
-    def tracker_store_testing_start (self, confdir=None):
+        if (os.environ.has_key ("TRACKER_DB_ONTOLOGIES_DIR")):
+            del os.environ ["TRACKER_DB_ONTOLOGIES_DIR"]
+
+    def tracker_store_testing_start (self, confdir=None, ontodir=None):
         """
         Stops any previous instance of the store, calls set_up_environment,
         and starts a new instances of the store
         """
         self.__stop_tracker_processes ()
-        self.set_up_environment (confdir)
+        self.set_up_environment (confdir, ontodir)
         
-        self.tracker_running = self.__start_tracker_store ()
+        self.store = TrackerStoreLifeCycle ()
+        self.store.start ()
+
+    def tracker_store_start (self):
+        self.store.start ()
+
+    def tracker_store_restart_with_new_ontologies (self, ontodir):
+        self.store.stop ()
+        if ontodir:
+            print "[Conf] Setting %s - %s" % ("TRACKER_DB_ONTOLOGIES_DIR", ontodir)
+            os.environ ["TRACKER_DB_ONTOLOGIES_DIR"] = ontodir
+        try:
+            self.store.start ()
+        except dbus.DBusException, e:
+            raise UnableToBootException ("Unable to boot the store \n(" + str(e) + ")")
 
     def tracker_store_brutal_restart (self):
-        self.tracker_running.kill ()
-        time.sleep (1)
-        self.tracker_running = self.__start_tracker_store ()
-        time.sleep (1)
+        self.store.kill ()
+        self.store.start ()
 
-    def tracker_store_remove_dbs (self):
-        db_location = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker")
-        shutil.rmtree (db_location)
-        os.mkdir (db_location)
+    def tracker_store_prepare_journal_replay (self):
+        db_location = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker", "meta.db")
+        os.unlink (db_location)
+
+        lockfile = os.path.join (TEST_ENV_VARS ['XDG_DATA_HOME'], "tracker", "data", ".ismeta.running")
+        f = open (lockfile, 'w')
+        f.write (" ")
+        f.close ()
 
     def tracker_store_corrupt_dbs (self):
         db_path = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker", "meta.db")
@@ -232,11 +345,18 @@ class TrackerSystemAbstraction:
         shutil.rmtree (db_location)
         os.mkdir (db_location)
 
+    def tracker_store_remove_dbs (self):
+        db_location = os.path.join (TEST_ENV_VARS ['XDG_CACHE_HOME'], "tracker")
+        shutil.rmtree (db_location)
+        os.mkdir (db_location)
+        
+
     def tracker_store_testing_stop (self):
         """
         Stops a running tracker-store and unset all the XDG_*_HOME vars
         """
-        self.__stop_tracker_processes ()
+        assert self.store
+        self.store.stop ()
         self.unset_up_environment ()
 
 
@@ -246,7 +366,11 @@ class TrackerSystemAbstraction:
         and starts a new instance of the store and miner-fs
         """
         self.__stop_tracker_processes ()
-        self.set_up_environment (confdir)
+        self.set_up_environment (confdir, None)
+
+        # Start also the store. DBus autoactivation ignores the env variables.
+        self.store = TrackerStoreLifeCycle ()
+        self.store.start ()
 
         self.miner_fs = TrackerMinerFsLifeCycle ()
         self.miner_fs.start ()
@@ -256,7 +380,6 @@ class TrackerSystemAbstraction:
         Copy the files physically in the filesyste and wait for the miner to complete the work
         """
         self.miner_fs.wait_for_idle (timeout)
-        time.sleep (1)
 
 
     def tracker_miner_fs_testing_stop (self):
@@ -264,6 +387,8 @@ class TrackerSystemAbstraction:
         Stops the miner-fs and store running and unset all the XDG_*_HOME vars
         """
         self.miner_fs.stop ()
+        self.store.stop ()
+        
         self.__stop_tracker_processes ()
         self.unset_up_environment ()
 
@@ -282,14 +407,6 @@ class TrackerSystemAbstraction:
     #
     # Private API
     #
-    def __start_tracker_store (self):
-        tracker_binary = os.path.join (cfg.EXEC_PREFIX, "tracker-store")
-        tracker = [tracker_binary]
-        # The env variables can be passed as parameters!
-        FNULL = open('/dev/null', 'w')
-        return subprocess.Popen (tracker, stdout=FNULL, stderr=FNULL)
-
-
     def __stop_tracker_processes (self):
         control_binary = os.path.join (cfg.BINDIR, "tracker-control")
         FNULL = open('/dev/null', 'w')
