@@ -46,18 +46,18 @@ test_blank (void)
 	                           NULL,
 	                           NULL,
 	                           FALSE,
+	                           100,
+	                           100,
 	                           NULL,
 	                           NULL,
 	                           NULL);
 
 	/* perform update in transaction */
 
-	tracker_data_begin_db_transaction ();
 	updates = tracker_data_update_sparql_blank (
 	                                            "INSERT { _:foo a rdfs:Resource } "
 	                                            "INSERT { _:foo a rdfs:Resource . _:bar a rdfs:Resource } ",
 	                                            &error);
-	tracker_data_commit_db_transaction ();
 	g_assert_no_error (error);
 
 	g_assert_cmpint (updates->len, ==, 2);
@@ -78,10 +78,6 @@ test_blank (void)
 
 	/* cleanup */
 
-	g_hash_table_unref (blank_nodes[0]);
-	g_hash_table_unref (blank_nodes[1]);
-	g_ptr_array_free (updates->pdata[0], TRUE);
-	g_ptr_array_free (updates->pdata[1], TRUE);
 	g_ptr_array_free (updates, TRUE);
 
 	tracker_data_manager_shutdown ();
